from setuptools import setup, find_packages

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup(
    name='my_quart_app',
    version='1.1.1',
    packages=find_packages(),
    include_package_data=True,
    install_requires=requirements,
    # entry_points={
    #     'console_scripts': [
    #         'my_quart_app = app.run_server:main',
    #     ]
    # },
)